#include "board.h"

using std::vector;

using namespace std;

const vector<vector<Cell>> Board::BOARD_1 =
{
    { BLOCKED, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY },
    { EMPTY, BLOCKED, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, BLOCKED, EMPTY, EMPTY },
    { EMPTY, EMPTY, BLOCKED, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY },
    { EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, BLOCKED, EMPTY, EMPTY },
    { EMPTY, EMPTY, EMPTY, EMPTY, BLOCKED, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY },
    { EMPTY, EMPTY, BLOCKED, EMPTY, EMPTY, BLOCKED, EMPTY, BLOCKED, EMPTY, EMPTY },
    { EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, BLOCKED, EMPTY, EMPTY, EMPTY },
    { EMPTY, EMPTY, BLOCKED, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY },
    { EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY },
    { EMPTY, EMPTY, BLOCKED, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, BLOCKED }
};

const vector<vector<Cell>> Board::BOARD_2 =
{
    { BLOCKED, BLOCKED, BLOCKED, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY },
    { EMPTY, BLOCKED, BLOCKED, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY },
    { EMPTY, BLOCKED, BLOCKED, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY },
    { EMPTY, EMPTY, EMPTY, EMPTY, BLOCKED, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY },
    { EMPTY, EMPTY, EMPTY, EMPTY, BLOCKED, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY },
    { EMPTY, BLOCKED, BLOCKED, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY },
    { EMPTY, EMPTY, EMPTY, EMPTY, BLOCKED, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY },
    { EMPTY, BLOCKED, BLOCKED, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY },
    { EMPTY, BLOCKED, BLOCKED, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY },
    { EMPTY, BLOCKED, BLOCKED, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY }
};

Board::Board()
{
    board = new std::vector<std::vector<Cell>>[1];

    for (int i = 0; i < 10; i++)
    {
        vector<Cell> row;
        for (int j = 0; j < 10; j++)
        {
            row.push_back(Cell::EMPTY);
        }

        (*board).push_back(row);
    }
}

Board::~Board()
{
    delete board;
}

void Board::load(int boardId)
{
    if(boardId == 1)
        (*board) = Board::BOARD_1;
    else if (boardId == 2)
        (*board) = Board::BOARD_2;
}

void Board::generate(int dimension, double probability)
{
    board = new std::vector<std::vector<Cell>>[1];

    for (int i = 0; i < dimension; i++)
    {
        vector<Cell> row;
        for (int j = 0; j < dimension; j++)
        {
            if ((rand() % 100) <= (probability * 100))
                row.push_back(Cell::BLOCKED);
            else
                row.push_back(Cell::EMPTY);
            
        }

        (*board).push_back(row);
    }
}

bool Board::placePlayer(Position position)
{
    int dims = (*board).size();

    if(position.x < 0 || position.y < 0 || position.x >= dims || position.y >= dims)
        return false;
    
    if ((*board)[position.x][position.y] == Cell::EMPTY)
        return true;

    return false;
}

PlayerMove Board::movePlayerForward(Player* player)
{
    Position position = player->getNextForwardPosition();
    int dims = (*board).size();

    if (placePlayer(position))
    {
        player->updatePosition(position);
        return PlayerMove::PLAYER_MOVED;
    }
    else if (position.x < 0 || position.y < 0 || position.x >= dims || position.y >= dims)
        return PlayerMove::OUTSIDE_BOUNDS;
    else
        return PlayerMove::CELL_BLOCKED;
}

void Board::display(Player* player)
{
    cout << endl << endl;

    int dims = (*board).size();

    cout << LINE_OUTPUT << " " << LINE_OUTPUT;
    for (int i = 0; i < dims; i++)
    {
        cout << (i % 10) << LINE_OUTPUT;
    }
    cout << endl;

    for (int i = 0; i < dims; i++)
    {
        cout << LINE_OUTPUT << (i % 10) << LINE_OUTPUT;
        
        for (int j = 0; j < dims; j++)
        {
            if (i == player->position.x && j == player->position.y)
            {
                player->displayDirection();
                cout << LINE_OUTPUT;
            }
            else if((*board)[i][j] == Cell::EMPTY)
                cout << EMPTY_OUTPUT << LINE_OUTPUT;
            else if ((*board)[i][j] == Cell::BLOCKED)
                cout << BLOCKED_OUTPUT << LINE_OUTPUT;            
        }

        cout << endl;

    }
}


