#include "game.h"

using namespace std;

Game::Game()
{
    board = nullptr;
    player = nullptr;
}

Game::~Game()
{
    if(board != nullptr)
        delete board;
    if (player != nullptr)
        delete player;
}


void Game::start()
{
    loadBoard();
    initializePlayer();
    play();
}

bool Game::loadBoard()
{
    board = new Board();
    return true;
}

bool Game::initializePlayer()
{
    player = new Player();
    return true;
}

void Game::play()
{
    string input;

    while (true)
    {
        board->display(player);

        cout << "\nYou can use the following commands to play the game:";
        cout << "\ngenerate <d>,<p>";
        cout << "\n\td: the dimension of the board to be generated";
        cout << "\n\tp: the probability of the block on board to be generated randomly";
        cout << "\ninit <x>,<y>,<direction>";
        cout << "\n\tx: horizontal position of the car on the board (between 0 & 9)";
        cout << "\n\ty: vertical position of the car on the board (between 0 & 9)";
        cout << "\n\tdirection: direction of the car\' movement (north, east, south, west)";
        cout << "\nforward (or f)";
        cout << "\nturn_left  (or l)";
        cout << "\nturn_right (or r)";
        cout << "\nquit";
        cout << "\n";

        input = Helper::readInput();

        vector<string> tokens;
        Helper::splitString(input, tokens, " ");

        if (tokens.size() >= 1)
        {
            if (tokens[0].compare(COMMAND_LOAD) == 0 && tokens.size() >= 2) {
                board->load(stoi(tokens[1]));
            }
            else if (tokens[0].compare(COMMAND_GENERATE_RANDOM) == 0 && tokens.size() >= 2) {
                vector<string> parameters;
                Helper::splitString(tokens[1], parameters, ",");

                if (parameters.size() >= 2)
                {
                    board->generate(stoi(parameters[0]), stod(parameters[1]));
                }
            }
            else if (tokens[0].compare(COMMAND_INIT) == 0 && tokens.size() >= 2) {
                vector<string> parameters;
                Helper::splitString(tokens[1], parameters, ",");

                if (parameters.size() >= 3)
                {
                    if (parameters[2].compare(DIRECTION_NORTH) == 0)
                        this->player->initialisePlayer(new Position(stoi(parameters[1]), stoi(parameters[0])), Direction::NORTH);
                    if (parameters[2].compare(DIRECTION_EAST) == 0)
                        this->player->initialisePlayer(new Position(stoi(parameters[1]), stoi(parameters[0])), Direction::EAST);
                    if (parameters[2].compare(DIRECTION_SOUTH) == 0)
                        this->player->initialisePlayer(new Position(stoi(parameters[1]), stoi(parameters[0])), Direction::SOUTH);
                    if (parameters[2].compare(DIRECTION_WEST) == 0)
                        this->player->initialisePlayer(new Position(stoi(parameters[1]), stoi(parameters[0])), Direction::WEST);
                }
            }
            else if (tokens[0].compare(COMMAND_FORWARD) == 0 || tokens[0].compare(COMMAND_FORWARD_SHORTCUT) == 0) {
                PlayerMove move = board->movePlayerForward(this->player);

                if (move == PlayerMove::PLAYER_MOVED) {
                    this->player->moves++;
                }
                else if (move == PlayerMove::CELL_BLOCKED) {
                    cout << "Blocked" << endl;
                }
                else if (move == PlayerMove::OUTSIDE_BOUNDS) {
                    cout << "The car is at edge of the board and cannot move further in that direction" << endl;
                }
            }
            else if (tokens[0].compare(COMMAND_TURN_LEFT) == 0 || tokens[0].compare(COMMAND_TURN_LEFT_SHORTCUT) == 0) {
                this->player->turnDirection(TurnDirection::TURN_LEFT);
            }
            else if (tokens[0].compare(COMMAND_TURN_RIGHT) == 0 || tokens[0].compare(COMMAND_TURN_RIGHT_SHORTCUT) == 0) {
                this->player->turnDirection(TurnDirection::TURN_RIGHT);
            }
            else if (tokens[0].compare(COMMAND_QUIT) == 0) {
                cout << "Total player moves : " << this->player->moves << endl;
                break;
            }
        }
    }
}